package com.chelvc.framework.database.support;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import org.apache.ibatis.annotations.Param;

/**
 * 数据模型增强操作接口，忽略字段验证策略配置
 *
 * @author Woody
 * @date 2024/4/3
 */
public interface EnhanceMapper<T> extends BaseMapper<T> {
    /**
     * 创建或忽略实体
     *
     * @param entity 实体对象实例
     * @return 影响行数
     */
    int createIgnore(T entity);

    /**
     * 创建或修改实体
     *
     * @param entity 实体对象实例
     * @param update 数据更新对象实例
     * @return 影响行数
     */
    int createUpdate(@Param("entity") T entity, @Param("update") Updates.Update<T> update);

    /**
     * 批量创建或忽略实体
     *
     * @param entities 实体对象实例集合
     * @return 影响行数
     */
    int batchCreateIgnore(Collection<T> entities);

    /**
     * 批量创建或修改实体
     *
     * @param entities 实体对象实例集合
     * @param update   数据更新对象实例
     * @return 影响行数
     */
    int batchCreateUpdate(@Param("entities") Collection<T> entities, @Param("update") Updates.Update<T> update);

    /**
     * 查询对象列表
     *
     * @param select  查询列
     * @param wrapper 查询条件
     * @return 数据对象列表
     */
    List<T> getObjects(@Param("select") Selects.Select<T> select, @Param(Constants.WRAPPER) Wrapper<T> wrapper);

    /**
     * 查询对象键/值映射表
     *
     * @param select  查询列
     * @param wrapper 查询条件
     * @return 键/值映射表列表
     */
    List<Map<String, Object>> getMaps(@Param("select") Selects.Select<T> select,
                                      @Param(Constants.WRAPPER) Wrapper<T> wrapper);

    /**
     * 查询对象分页
     *
     * @param page    分页信息
     * @param select  查询列
     * @param wrapper 查询条件
     * @param <E>     分页类型
     * @return 对象分页信息
     */
    <E extends IPage<T>> E getPage(E page, @Param("select") Selects.Select<T> select,
                                   @Param(Constants.WRAPPER) Wrapper<T> wrapper);

    /**
     * 查询对象键/值映射表分页
     *
     * @param page    分页信息
     * @param select  查询列
     * @param wrapper 查询条件
     * @param <E>     分页类型
     * @return 键/值映射表分页信息
     */
    <E extends IPage<Map<String, Object>>> E getMapsPage(E page, @Param("select") Selects.Select<T> select,
                                                         @Param(Constants.WRAPPER) Wrapper<T> wrapper);
}
